@@ -18,6 +18,7 @@
 #  undef MOUSE_MOVED
 #endif
 #include "colors.h"
+#include "display.h"
 
 void initcolor(void)
 {
@@ -56,11 +57,14 @@ void colorbox(WINDOW *win, chtype color, int hasbox)
 #ifndef PDCURSES
     int maxx;
 #endif
-    chtype attr = color & A_ATTR;  /* extract Bold, Reverse, Blink bits */
+#ifdef A_COLOR
+    chtype attr;
+#endif
 
     setcolor(win, color);
 
 #ifdef A_COLOR
+    attr = color & A_ATTR;  /* extract Bold, Reverse, Blink bits */
     if (has_colors())
         wbkgd(win,
             COLOR_PAIR(color & A_CHARTEXT) | (attr & ~A_REVERSE));
@@ -68,7 +72,7 @@ void colorbox(WINDOW *win, chtype color, int hasbox)
 #endif
         wbkgd(win, color);
 
-    werase(win);
+    WCD_WCLEAR(win);
 
 #ifdef PDCURSES
     maxy = getmaxy(win);
