@@ -1,5 +1,5 @@
 /*
-Copyright (C) 1997-2014 Erwin Waterlander
+Copyright (C) 1997-2016 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,6 +21,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <string.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <assert.h>
 #include "wcd.h"
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
 #ifndef __USE_XOPEN
@@ -44,7 +45,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 #ifdef WCD_ANSI
-/* wide char to UTF-8 */
+/* wide char to ANSI */
 size_t wcstoansi(char *mbstr, const wchar_t *wcstr, int len)
 {
    return((size_t)(WideCharToMultiByte(CP_ACP, 0, wcstr, -1, mbstr, len, NULL, NULL) -1));
@@ -116,11 +117,13 @@ void wcd_printf( const char* format, ... ) {
     * in Windows Unicode mode all treedata files are written in UTF-8 format.
     * Print to buffer (UTF-8) */
    vsnprintf( buf, sizeof(buf), formatmbs, args);
+   buf[sizeof(buf)-1] = '\0';
     /* Convert UTF-8 buffer to wide characters, and print to console. */
    if (MultiByteToWideChar(CP_UTF8,0, buf, -1, wstr, DD_MAXPATH) > 0  )
 #  else
    /* Everything is in ANSI code page */
    vsnprintf( buf, sizeof(buf), format, args);
+   buf[sizeof(buf)-1] = '\0';
    if (MultiByteToWideChar(CP_ACP,0, buf, -1, wstr, DD_MAXPATH) > 0  )
 #  endif
       WriteConsoleW(stduit, wstr, wcslen(wstr), NULL, NULL);
@@ -149,10 +152,10 @@ void wcd_printf( const char* format, ... ) {
 #if (defined(WCD_USECONIO) || defined(WCD_USECURSES))
 size_t str_columns (char *s)
 {
+   assert(s);
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
    static wchar_t wstr[DD_MAXPATH];
    size_t i;
-   int j;
 
    /* convert to wide characters. i = nr. of characters */
    i= MBSTOWCS(wstr,s,(size_t)DD_MAXPATH);
@@ -160,7 +163,7 @@ size_t str_columns (char *s)
       return(strlen(s));
    else
    {
-      j = wcd_wcswidth(wstr,(size_t)DD_MAXPATH);
+      int j = wcd_wcswidth(wstr,(size_t)DD_MAXPATH);
       /* j =  nr. of columns */
       if ( j < 0)
          return(strlen(s));
@@ -195,7 +198,6 @@ void ssort (nameset list, int left, int right)
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
  static wchar_t wstr_left[DD_MAXPATH];
  static wchar_t wstr_right[DD_MAXPATH];
- size_t len1,len2;
 #endif
 
   if (left >= right) return; /* fewer than 2 elements */
@@ -206,8 +208,8 @@ void ssort (nameset list, int left, int right)
   for (i = left+1; i <=right; i++)
   {
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-   len1 = MBSTOWCS(wstr_left, list->array[left],(size_t)DD_MAXPATH);
-   len2 = MBSTOWCS(wstr_right,list->array[i],(size_t)DD_MAXPATH);
+   size_t len1 = MBSTOWCS(wstr_left, list->array[left],(size_t)DD_MAXPATH);
+   size_t len2 = MBSTOWCS(wstr_right,list->array[i],(size_t)DD_MAXPATH);
    if ((len1 == (size_t)(-1)) || (len2 == (size_t)(-1)))
    {
       /* Erroneous multi-byte sequence */
@@ -252,7 +254,7 @@ void sort_list(nameset list)
 #if (defined(WCD_USECONIO) || defined(WCD_USECURSES))
 size_t maxLength(nameset list)
 {
-   size_t i, len, maxlen = 0;
+   size_t i, maxlen = 0;
 
    if (list == NULL)
    {
@@ -262,6 +264,7 @@ size_t maxLength(nameset list)
 
    for (i=0;i<list->size;i++)
    {
+    size_t len;
     if( (len=str_columns(list->array[i])) > maxlen)
        maxlen=len;
    }
@@ -272,7 +275,7 @@ size_t maxLength(nameset list)
 }
 size_t maxLengthStack(WcdStack s)
 {
-   size_t i, len,maxlen = 0;
+   size_t i, maxlen = 0;
 
    if (s == NULL)
    {
@@ -282,6 +285,7 @@ size_t maxLengthStack(WcdStack s)
 
    for (i=0;i<s->size;i++)
    {
+    size_t len;
     if( (len=str_columns(s->dir[i])) > maxlen)
        maxlen=len;
    }
@@ -319,19 +323,19 @@ size_t maxLengthStack(WcdStack s)
 void printLine(nameset n, int i, int y, int xoffset, int *use_numbers, int screenWidth)
 {
    wcd_uchar *s;
-   int len, j, nr_offset;
 
    s = n->array[i];
 
    if (s != NULL)
    {
-      len = strlen(s);
+      int len = strlen(s);
+      int nr_offset;
       if (*use_numbers == 0)
          nr_offset = 2;
       else
          nr_offset = 3;
 
-      for(j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(screenWidth-1));j++)
+      for(int j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(screenWidth-1));j++)
       {
          cprintf("%c",s[j]);
       }
@@ -341,18 +345,19 @@ void printLine(nameset n, int i, int y, int xoffset, int *use_numbers, int scree
 void printStackLine(WcdStack ws, int i, int y, int xoffset, int *use_numbers, int screenWidth)
 {
    wcd_uchar *s;
-   int len, j, nr_offset;
 
    s = ws->dir[i];
 
    if (s != NULL)
    {
-      len = strlen(s);
+      int len = strlen(s);
+      int nr_offset;
       if (*use_numbers == 0)
          nr_offset = 2;
       else
          nr_offset = 3;
 
+      int j;
       for(j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(screenWidth-1));j++)
       {
          cprintf("%c",s[j]);
@@ -447,7 +452,7 @@ char buf[WCD_MAX_INPSTR];
    else
       if (ws != NULL)   /* stack */
       {
-         if( ((ws->size) <= 0) || ((ws->size) > ws->maxsize) || ((ws->size) > ws->maxsize) )
+         if( ((ws->size) <= 0) || ((ws->size) > ws->maxsize) )
             return(WCD_ERR_LIST); /* in case stack file was corrupt */
          else
          {
@@ -490,8 +495,6 @@ char buf[WCD_MAX_INPSTR];
       else
          return(WCD_ERR_LIST);
 
-   top = 0;
-
    bottom = size -1;
    top = size - lines_per_page;
    if (top < 0)
@@ -837,7 +840,6 @@ void displayRefresh(int init);
 #  define CAN_RESIZE 0
 #endif
 
-#if CAN_RESIZE
 /* The most portable way of resizing seems to be
  * to just rebuild the windows from scratch */
 void displayResize()
@@ -865,14 +867,8 @@ void displayResize()
    delwin(wcd_display.inputWin);
 
    /* create new windows */
-   wcd_display.scrollWin = newpad(wcd_display.scrollWinHeight,COLS);
-
-   if (COLS < INPUT_WIN_LEN)
-      wcd_display.inputWinLen = INPUT_WIN_LEN;
-   else
-      wcd_display.inputWinLen = COLS;
-
-   wcd_display.inputWin = newpad(INPUT_WIN_HEIGHT,wcd_display.inputWinLen);
+   wcd_display.scrollWin = newwin(wcd_display.scrollWinHeight,COLS,0,0);
+   wcd_display.inputWin = newwin(INPUT_WIN_HEIGHT,COLS,wcd_display.scrollWinHeight,0);
 
    scrollok(wcd_display.scrollWin, TRUE);
 
@@ -880,6 +876,7 @@ void displayResize()
    displayRefresh(1);
 }
 
+#if CAN_RESIZE
 void signalSigwinchDisplay (int sig)
 {
   displayResize ();
@@ -912,22 +909,19 @@ void wcd_mvwaddstr(WINDOW *win, int x, int y, char *str)
 void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbers)
 {
    wcd_uchar *s;
-   size_t len;
-   int j, nr_offset;
-#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-   static wchar_t wstr[DD_MAXPATH];
-   int width, c;
-#endif
 
    s = (wcd_uchar *)n->array[i];
 
    if (s != NULL)
    {
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-      len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
+      static wchar_t wstr[DD_MAXPATH];
+      size_t len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
 #else
-      len = strlen((char *)s);
+      size_t len = strlen((char *)s);
+      int j;
 #endif
+      int nr_offset;
       if (*use_numbers == 0)
          nr_offset = 2;
       else
@@ -940,14 +934,15 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
       {
          /* Erroneous multi-byte sequence */
          /* Try 8 bit characters */
+         int j;
          len = strlen((char *)s);
          for(j=xoffset;(j<(int)len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
          {
             waddch(win,(chtype)s[j]);
          }
       } else {
+         int width, c, j = 0;
          c = 0; /* count characters with width > 0 from beginning of string. */
-         j = 0;
          while ((j<(int)len)&&(c<xoffset))
          {
             if (wcd_wcwidth(wstr[j]) != 0 )
@@ -956,7 +951,7 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
          }
 #if defined(_WIN32) || defined(__CYGWIN__)
          /* Line starts with a trailing surrogate character, insert a space */
-	 if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
+         if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
            wstr[j] = ' ';
 #endif
          while ((j<(int)len)&&(wcd_wcwidth(wstr[j]) == 0 ))   /* Skip combining characters */
@@ -981,22 +976,19 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
 void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *use_numbers)
 {
    wcd_uchar *s;
-   size_t len;
-   int j, nr_offset;
-#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-   static wchar_t wstr[DD_MAXPATH];
-   int width, c;
-#endif
 
    s = (wcd_uchar *)ws->dir[i];
 
    if (s != NULL)
    {
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-      len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
+      static wchar_t wstr[DD_MAXPATH];
+      size_t len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
 #else
-      len = strlen((char *)s);
+      size_t len = strlen((char *)s);
+      int j;
 #endif
+      int nr_offset;
       if (*use_numbers == 0)
          nr_offset = 2;
       else
@@ -1009,6 +1001,7 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
       {
          /* Erroneous multi-byte sequence */
          /* Try 8 bit characters */
+         int j;
          len = strlen((char *)s);
          for(j=xoffset;(j<(int)len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
          {
@@ -1017,8 +1010,8 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
          if ((i == ws->current) && ((nr_offset+j-xoffset+2)<(COLS-1)))
             wprintw(win," *");
       } else {
+         int width, c, j = 0;
          c = 0; /* count characters with width > 0 from beginning of string. */
-         j = 0;
          while ((j<(int)len)&&(c<xoffset))
          {
             if (wcd_wcwidth(wstr[j]) != 0 )
@@ -1027,7 +1020,7 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
          }
 #if defined(_WIN32) || defined(__CYGWIN__)
          /* Line starts with a trailing surrogate character, insert a space */
-	 if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
+         if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
            wstr[j] = ' ';
 #endif
          while ((j<(int)len)&&(wcd_wcwidth(wstr[j]) == 0 ))   /* Skip combining characters */
@@ -1100,7 +1093,7 @@ void print_list_stack(WINDOW *scrollWin, int lines_per_page,int line, WcdStack w
 
 void print_list(WINDOW *scrollWin, int lines_per_page,int line, nameset list, WcdStack ws, int start, int top, int bottom, int use_numbers, int xoffset)
 {
-   werase(scrollWin);
+   WCD_WCLEAR(scrollWin);
    if (list != NULL)
       print_list_normal(scrollWin,lines_per_page,line,list,top,bottom,use_numbers,xoffset);
    else
@@ -1117,8 +1110,8 @@ void displayRefresh(int init)
   int page, pageoffset, offset, n;
   char buf[WCD_MAX_INPSTR];
 
-  werase(wcd_display.scrollWin);
-  werase(wcd_display.inputWin);
+  WCD_WCLEAR(wcd_display.scrollWin);
+  WCD_WCLEAR(wcd_display.inputWin);
 
 
  if (init || (wcd_display.bottom >= wcd_display.size) || (wcd_display.size <= wcd_display.lines_per_page)) {
@@ -1163,13 +1156,13 @@ void displayRefresh(int init)
    n = (int)str_columns(wcd_display.number_str) ;
    wmove (wcd_display.inputWin, 2, offset + n);
 
-   prefresh(wcd_display.scrollWin,0,0,0,0,wcd_display.scrollWinHeight-1,COLS-1);
-   prefresh(wcd_display.inputWin,0,0,wcd_display.scrollWinHeight,0,wcd_display.scrollWinHeight+INPUT_WIN_HEIGHT-1,COLS-1);
+   wrefresh(wcd_display.scrollWin);
+   wrefresh(wcd_display.inputWin);
 
 }
 void displayHelp(WINDOW *win, int height)
 {
-   werase(win);
+   WCD_WCLEAR(win);
    if (height < 17)
       wcd_mvwaddstr(win,0,0,_("Screenheight must be > 20 for help."));
    else
@@ -1182,11 +1175,12 @@ void displayHelp(WINDOW *win, int height)
       wcd_mvwaddstr(win, 5,0,_("> or ]            scroll 10 right"));
       wcd_mvwaddstr(win, 6,0,_("CTRL-a or <HOME>  scroll to beginning"));
       wcd_mvwaddstr(win, 7,0,_("CTRL-e or <END>   scroll to end"));
-      wcd_mvwaddstr(win, 8,0,_("CTRL-c or <Esc>   abort"));
-      wcd_mvwaddstr(win, 9,0,_("<Enter>           abort"));
-      wcd_mvwaddstr(win,11,0,_("Press any key."));
+      wcd_mvwaddstr(win, 8,0,_("CTRL-l or F5      redraw screen"));
+      wcd_mvwaddstr(win, 9,0,_("CTRL-c or <Esc>   abort"));
+      wcd_mvwaddstr(win,10,0,_("<Enter>           abort"));
+      wcd_mvwaddstr(win,12,0,_("Press any key."));
    }
-   prefresh(win,0,0,0,0,height-1,COLS-1);
+   wrefresh(win);
    getch();
 
 }
@@ -1297,7 +1291,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 
    refresh();
 
-   wcd_display.scrollWin = newpad(wcd_display.scrollWinHeight,COLS);
+   wcd_display.scrollWin = newwin(wcd_display.scrollWinHeight,COLS,0,0);
    if (wcd_display.scrollWin == NULL)
    {
       endwin();
@@ -1310,13 +1304,8 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 
    scrollok(wcd_display.scrollWin, TRUE);
 
+   wcd_display.inputWin = newwin(INPUT_WIN_HEIGHT,COLS,wcd_display.scrollWinHeight,0);
 
-   if (COLS < INPUT_WIN_LEN)
-      wcd_display.inputWinLen = INPUT_WIN_LEN;
-   else
-      wcd_display.inputWinLen = COLS;
-
-   wcd_display.inputWin = newpad(INPUT_WIN_HEIGHT,wcd_display.inputWinLen);
    if (wcd_display.inputWin == NULL)
    {
       endwin();
@@ -1413,6 +1402,10 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
       case '?':
             displayHelp(wcd_display.scrollWin, wcd_display.scrollWinHeight);
          break;
+      case KEY_F (5):
+      case Key_CTRL ('l'):
+            displayResize ();
+         break;
      case 3:  /* Control-C */
      case 27: /* Escape */
       c = 13;
@@ -1431,7 +1424,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
             wcd_display.number_str[n] = '\0';
      break;
      default:
-         if (( c >= '0') && ( c <= '9') && (n < WCD_MAX_INPSTR)) /* numbers */
+         if (( c >= '0') && ( c <= '9') && (n < (WCD_MAX_INPSTR -1))) /* numbers */
          {
             wcd_display.number_str[n] = (char)c;
             n++;
@@ -1492,7 +1485,6 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
 {
    int i;
-   size_t ii;
    int k, start, j;
 
    if (list != NULL) /* normal list */
@@ -1501,11 +1493,13 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
 
       if ( use_stdout & WCD_STDOUT_DUMP )
       {
+         size_t ii;
          for (ii=0;ii<list->size;ii++)
             wcd_printf("%s\n", list->array[ii]);
       }
       else
       {
+         size_t ii;
          for (ii=0;ii<list->size;ii++)
             wcd_printf("%lu  %s\n",(unsigned long)(ii+1),list->array[ii]);
       }
@@ -1562,7 +1556,7 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
             if ( use_stdout & WCD_STDOUT_DUMP )
               return(WCD_ERR_LIST);
 
-	    printf("\n");
+            printf("\n");
             printf(_("Please choose one (<Enter> to abort): "));
             i = wcd_get_int();
 
